package de.dhsn.oop.ui;

import de.dhsn.oop.data.TodoList;
import de.dhsn.oop.ui.UIOverrides.FlatButton;
import de.dhsn.oop.ui.helpers.TodoListListModel;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

/**
 * MainWindow is the main Class started by the static void main function after initializing the datastructure
 */
public class MainWindow extends JFrame implements ActionListener, ListSelectionListener {
    private FlatButton addNewList;
    private List<TodoList> lists;
    private JList<String> list;

    /**
     * Main window constructor, takes a (potentially empty) list of TodoList
     * this list of lists is supposed to keep all application state for easy file save and open operations
     *
     * @param lists A list of TodoList
     * @throws HeadlessException
     */
    public MainWindow(List<TodoList> lists) throws HeadlessException {
        super();
        this.lists = lists;
        setBackground(Color.WHITE);
        Container cp = getContentPane();
        cp.removeAll();
        setTitle("Todo List");
        setSize(new Dimension(800, 600));
        setMinimumSize(new Dimension(200, 150));
        setLayout(new BorderLayout());

        list = new JList<>();
        list.setModel(new TodoListListModel(lists));
        list.addListSelectionListener(this);
        cp.add(list, BorderLayout.NORTH);

        addNewList = new FlatButton("Neue Liste Anlegen");
        addNewList.addActionListener(this);
        addNewList.setBackground(Color.WHITE);
        cp.add(addNewList, BorderLayout.SOUTH);
        cp.setBackground(Color.WHITE);

        pack();
        setVisible(true);

        this.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                System.out.println("exit test");
                //save to file
                try (BufferedWriter writer = new BufferedWriter(new FileWriter(new File("./test.json")))) {
                    writer.write(lists + "");
                } catch (IOException ex) {
                    throw new RuntimeException(ex);
                }
                System.out.println(lists);
                System.exit(0);
            }
        });
    }

    /**
     * listener function for the add new list button on the main page
     *
     * @param e the event to be processed
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        new NewListDialog(this);
    }

    /**
     * callback function to be called with a title for a new list
     * this is called from NewListDialog when the user presses the save button
     *
     * @param title title to use for the new todolist
     * @return
     */
    public void nldCallback(String title) {
        lists.add(new TodoList(title));
        //overwrite listview contents
        list.setModel(new TodoListListModel(lists));
    }

    /**
     * handle click events on the list of list titles
     * displays the list in a new window
     *
     * @param e the event that characterizes the change.
     */
    @Override
    public void valueChanged(ListSelectionEvent e) {
        // two events are emitted - only execute the code once
        if (e.getValueIsAdjusting()) {
            //show new window for the selected list
            new TodoListView(lists.get(list.getSelectedIndex()), this);
        }
    }

    // todo fühlt sich nach nem reudigen hack an
    public void restartList(TodoList l){
        new TodoListView(l, this);
    }

    public void deleteList(TodoList l){
        lists.remove(l);
        list.setModel(new TodoListListModel(lists));
    }
}
